import { NetworkInterface, createNetworkInterface } from './networkInterface';
import { GraphQLResult, Document } from 'graphql';
import { createApolloStore, ApolloStore, createApolloReducer, ApolloReducerConfig } from './store';
import { QueryManager, WatchQueryOptions, ObservableQuery } from './QueryManager';
import { readQueryFromStore, readFragmentFromStore } from './data/readFromStore';
import { IdGetter } from './data/extensions';
export { createNetworkInterface, createApolloStore, createApolloReducer, readQueryFromStore, readFragmentFromStore };
export default class ApolloClient {
    networkInterface: NetworkInterface;
    store: ApolloStore;
    reduxRootKey: string;
    initialState: any;
    queryManager: QueryManager;
    reducerConfig: ApolloReducerConfig;
    constructor({networkInterface, reduxRootKey, initialState, dataIdFromObject}?: {
        networkInterface?: NetworkInterface;
        reduxRootKey?: string;
        initialState?: any;
        dataIdFromObject?: IdGetter;
    });
    watchQuery: (options: WatchQueryOptions) => ObservableQuery;
    query: (options: WatchQueryOptions) => Promise<GraphQLResult>;
    mutate: (options: {
        mutation: Document;
        variables?: Object;
    }) => Promise<GraphQLResult>;
    reducer(): Function;
    middleware: () => (store: ApolloStore) => (next: any) => (action: any) => any;
    initStore(): void;
    private setStore;
}
